﻿using Android.Util;
using Android.Views;
using System;
using OpenTK.Platform.Android;
using Android.Content;
using OpenTK;
using OpenTK.Graphics;
using Android.Runtime;
using System.Reflection;
using Engine.Shared;

namespace Engine.Android
{
    public class GameView : AndroidGameView
    {
        private GameController gameController;

        public GameView(Context context, IAttributeSet attrs)
            : base(context, attrs)
        {

        }

        public GameView(IntPtr handle, JniHandleOwnership transfer)
            : base(handle, transfer)
        {

        }

        public void Initialize(GameController gameController)
        {
            this.gameController = gameController;
        }

        protected override void CreateFrameBuffer()
        {
            ContextRenderingApi = GLVersion.ES3;
            string tag = "Engine";

            try
            {
                Log.Verbose(tag, "Loading with high quality settings");

                GraphicsMode = new GraphicsMode(new ColorFormat(32), 24, 0, 0);

                base.CreateFrameBuffer();

                return;
            }
            catch (Exception ex)
            {
                Log.Verbose(tag, "{0}", ex);
            }

            try
            {
                Log.Verbose(tag, "Loading with default settings");

                base.CreateFrameBuffer();

                return;
            }
            catch (Exception ex)
            {
                Log.Verbose(tag, "{0}", ex);
            }

            try
            {
                Log.Verbose(tag, "Loading with custom Android settings (low mode)");
                GraphicsMode = new AndroidGraphicsMode(16, 0, 0, 0, 0, false);

                base.CreateFrameBuffer();

                return;
            }
            catch (Exception ex)
            {
                Log.Verbose(tag, "{0}", ex);
            }

            try
            {
                Log.Verbose(tag, "Loading with no Android settings");
                GraphicsMode = new AndroidGraphicsMode(0, 4, 0, 0, 0, false);

                base.CreateFrameBuffer();

                return;
            }
            catch (Exception ex)
            {
                Log.Verbose(tag, "{0}", ex);
            }

            throw new Exception("Can't load OpenGL");
        }

        protected override void OnRenderFrame(FrameEventArgs e)
        {
            base.OnRenderFrame(e);

            MakeCurrent();

            gameController.Render();

            SwapBuffers();
        }

        protected override void OnLoad(EventArgs e)
        {
            if (!gameController.IsLoaded)
            {
                gameController.Load();
            }
        }

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);

            if (gameController.IsLoaded)
            {
                gameController.Resize(Width, Height);
            }
        }
    }
}
