﻿using OpenTK.Graphics.ES30;

namespace Engine.Shared
{
    public class GameController
    {
        public bool IsLoaded { get; set; } = false;

        public void Load()
        {
            // Return since state is only clear for OnStop, OnPause or low memory.
            // https://developer.android.com/reference/android/app/Activity#ActivityLifecycle
            if (IsLoaded)
            {
                return;
            }

            IsLoaded = true;

            GL.ClearColor(1f, 1f, 1f, 0f); // Use white as clear color
            GL.Enable(EnableCap.CullFace); // Activate differentiation which that only one side of faces is visible
            GL.CullFace(CullFaceMode.Back); // Don't draw backside of faces
            GL.Disable(EnableCap.DepthTest); // Enable for transparent textures
            GL.Disable(EnableCap.Dither); // Turn off color illusion
            GL.FrontFace(FrontFaceDirection.Ccw); // Polygons that are draw counter clock wise are pointing to the front
        }

        public void Render()
        {
            GL.Clear(ClearBufferMask.ColorBufferBit | ClearBufferMask.DepthBufferBit);
        }

        public void Resize(int width, int height)
        {
            if (width > 0 && height > 0)
            {
                GL.Viewport(0, 0, width, height);
            }
        }
    }
}
